/*
 * © 2017 LogMeIn, Inc. All Rights Reserved.
 * All rights reserved.
 * 
 * This software is distributed under the terms and conditions of the
 * LogMeIn SDK License Agreement. Please see file LICENSE for details.
 * 
 * Auto-generated file.
 */


using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using Newtonsoft.Json;

namespace LogMeIn.GoToTraining.Api.Common
{
    /// <summary>
    ///
    /// </summary>
    public class ApiInvoker
    {
        private static readonly ApiInvoker _instance = new ApiInvoker();
        private Dictionary<String, String> defaultHeaderMap = new Dictionary<String, String>
        {
            {"CALLING_CLIENT", "GoToTraining .NET SDK version 1.0.0"}
        };

        /// <summary>
        ///
        /// </summary>
        public static ApiInvoker GetInstance()
        {
            return _instance;
        }

        /// <summary>
        ///
        /// </summary>
        public void AddDefaultHeader(string key, string value)
        {
            defaultHeaderMap[key] = value;
        }

        /// <summary>
        ///
        /// </summary>
        public string EscapeString(string str)
        {
            return str;
        }

        private static string FixResponse(string json)
        {
            var rexDate = new Regex(@"(\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\+0{4})");

            json = rexDate.Replace(json, m =>
            {
                string match = m.ToString();
                return match.Substring(0, match.IndexOf(".")) + ".000Z";
            });

            var rexDateArray = new Regex(@"(\[\s*""\d{2}\/\d{2}\/\d{4} \d{2}:\d{2}:\d{2}""\s*,\s*""\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\+?0*Z?""\s*\])");

            json = rexDateArray.Replace(json, m =>
            {
                string match = m.ToString();
                int startPosition = match.IndexOf(",") + 1;

                return match.Substring(startPosition, match.IndexOf("]") - startPosition);
            });

            var rexMeetingId = new Regex(@"(""meetingId""\s*:\s*""[0-9]*-+[0-9-]*"")");

            json = rexMeetingId.Replace(json, m =>
            {
                string match = m.ToString();

                return match.Replace("-", "");
            });

            var rexDateCharsDayTwoDigits = new Regex(@"(.{3}\s.{3}\s\d{2}\s\d{2}:\d{2}:\d{2}\s\d{4})");

            json = rexDateCharsDayTwoDigits.Replace(json, m =>
            {
                string match = m.ToString();

                return DateTime.ParseExact(match, "ddd MMM d HH:mm:ss yyyy", CultureInfo.InvariantCulture).Stringify();
            });

            var rexDateCharsDayOneDigit = new Regex(@"(.{3}\s.{3}\s{2}\d{1}\s\d{2}:\d{2}:\d{2}\s\d{4})");

            json = rexDateCharsDayOneDigit.Replace(json, m =>
            {
                string match = m.ToString();

                return DateTime.ParseExact(match, "ddd MMM  d HH:mm:ss yyyy", CultureInfo.InvariantCulture).Stringify();
            });

            var rexDateCharsGlobal = new Regex(@"(.{3}\s.{3}\s{2}\d{2}\s\d{2}:\d{2}:\d{2}\s\d{4})");

            json = rexDateCharsGlobal.Replace(json, m =>
            {
                string match = m.ToString();

                return DateTime.ParseExact(match, "ddd MMM  dd HH:mm:ss yyyy", CultureInfo.InvariantCulture).Stringify();
            });

            return json;
        }

        /// <summary>
        ///
        /// </summary>
        public static object Deserialize(string json, Type type)
        {
            if (string.IsNullOrEmpty(json))
                return null;

            var settings = new JsonSerializerSettings {MissingMemberHandling = MissingMemberHandling.Ignore};

            try
            {
                return JsonConvert.DeserializeObject(json, type, settings);
            }
            catch (JsonSerializationException)
            {
                json = FixResponse(json);
                return JsonConvert.DeserializeObject(json, type, settings);
            }
            catch (JsonReaderException)
            {
                json = FixResponse(json);
                return JsonConvert.DeserializeObject(json, type, settings);
            }
            catch (IOException e)
            {
                throw new ApiException(e);
            }
        }

        /// <summary>
        ///
        /// </summary>
        public static string Serialize(object obj)
        {
            var settings = new JsonSerializerSettings {NullValueHandling = NullValueHandling.Ignore};

            try
            {
                return obj != null ? JsonConvert.SerializeObject(obj, settings) : null;
            }
            catch (Exception e)
            {
                throw new ApiException(e);
            }
        }

        /// <summary>
        ///
        /// </summary>
        public object InvokeAPI(string basePath, string path, string method, Dictionary<string, string> queryParams,
            object body, Dictionary<string, string> headerParams, Dictionary<string, object> formParams, Type responseType, string contentType)
        {
            if (responseType == typeof(byte[]))
                return InvokeAPI(basePath, path, method, true, queryParams, body, headerParams, formParams, contentType);

            string response = InvokeAPI(basePath, path, method, false, queryParams, body, headerParams, formParams, contentType) as string;
            return Deserialize(response, responseType);
        }

        private object InvokeAPI(string basePath, string path, string method, bool binaryResponse, Dictionary<string, string> queryParams, object body, Dictionary<string, string> headerParams, Dictionary<string, object> formParams, string contentType)
        {
            var b = new StringBuilder();

            foreach (var queryParamItem in queryParams)
            {
                var value = queryParamItem.Value;
                if (value == null)
                    continue;
                b.Append(b.ToString().Length == 0 ? "?" : "&");
                b.Append(EscapeString(queryParamItem.Key)).Append("=").Append(EscapeString(value));
            }

            var querystring = b.ToString();

            basePath = basePath.EndsWith("/") ? basePath.Substring(0, basePath.Length - 1) : basePath;

            var client = (HttpWebRequest)WebRequest.Create(basePath + path + querystring);
            client.Method = method;
            client.ContentType = contentType;

            byte[] formData = null;
            if (formParams.Count > 0)
            {
                if (client.ContentType == "application/x-www-form-urlencoded")
                {
                    var sb = new StringBuilder();
                    foreach (var kvp in formParams)
                    {
                        if (kvp.Value != null && kvp.Value.ToString() != "")
                        {
                            if (sb.Length > 0)
                                sb.Append("&");

                            try
                            {
                                sb.Append(Uri.EscapeDataString(kvp.Key)).Append("=").Append(Uri.EscapeDataString(kvp.Value.ToString()));
                            }
                            catch (Exception)
                            {
                                // move on to next
                            }
                        }
                    }

                    body = sb.ToString();
                    client.Accept = "application/json";
                }
                else
                {
                    string formDataBoundary = String.Format("----------{0:N}", Guid.NewGuid());
                    client.ContentType = "multipart/form-data; boundary=" + formDataBoundary;
                    formData = GetMultipartFormData(formParams, formDataBoundary);
                    client.ContentLength = formData.Length;
                }
            }
            else
            {
                client.Accept = "application/json";
            }

            foreach (var headerParamsItem in headerParams)
            {
                if (headerParamsItem.Key == "Accept" && !string.IsNullOrEmpty(headerParamsItem.Value))
                    client.Accept = headerParamsItem.Value;
                else
                    client.Headers[headerParamsItem.Key] = headerParamsItem.Value;
            }

            foreach (var defaultHeaderMapItem in defaultHeaderMap)
                client.Headers[defaultHeaderMapItem.Key] = defaultHeaderMapItem.Value;

            switch (method)
            {
                case "GET":
                case "HEAD":
                    break;
                case "POST":
                case "PUT":
                case "DELETE":
                    using (Stream requestStream = client.GetRequestStream())
                    {
                        if (formData != null)
                            requestStream.Write(formData, 0, formData.Length);

                        var swRequestWriter = new StreamWriter(requestStream);
                        swRequestWriter.Write(client.ContentType == "application/x-www-form-urlencoded" ? body : Serialize(body));
                        swRequestWriter.Close();
                    }
                    break;
                default:
                    throw new ApiException("Unexpected method " + method);
            }

            try
            {
                var webResponse = (HttpWebResponse)client.GetResponse();
                if (webResponse.StatusCode != HttpStatusCode.OK
                    && webResponse.StatusCode != HttpStatusCode.Created
                    && webResponse.StatusCode != HttpStatusCode.NoContent
                    && webResponse.StatusCode != HttpStatusCode.Accepted)
                {
                    webResponse.Close();
                    throw new ApiException((int)webResponse.StatusCode, webResponse.StatusDescription);
                }

                if (binaryResponse)
                {
                    using (var memoryStream = new MemoryStream())
                    {
                        webResponse.GetResponseStream().CopyTo(memoryStream);
                        return memoryStream.ToArray();
                    }
                }
                else
                {
                    using (var responseReader = new StreamReader(webResponse.GetResponseStream()))
                    {
                        var responseData = responseReader.ReadToEnd();
                        return responseData;
                    }
                }
            }
            catch (WebException ex)
            {
                var response = ex.Response as HttpWebResponse;
                int statusCode = 0;
                if (response != null)
                {
                    statusCode = (int)response.StatusCode;
                    response.Close();
                }
                throw new ApiException(statusCode, ex.Message);
            }
        }

        private static byte[] GetMultipartFormData(Dictionary<string, object> postParameters, string boundary)
        {
            Stream formDataStream = new System.IO.MemoryStream();
            bool needsCLRF = false;

            foreach (var param in postParameters)
            {
                // Thanks to feedback from commenters, add a CRLF to allow multiple parameters to be added.
                // Skip it on the first parameter, add it to subsequent parameters.
                if (needsCLRF)
                    formDataStream.Write(Encoding.UTF8.GetBytes("\r\n"), 0, Encoding.UTF8.GetByteCount("\r\n"));

                needsCLRF = true;

                if (param.Value is byte[])
                {
                    string postData = string.Format("--{0}\r\nContent-Disposition: form-data; name=\"{1}\"; filename=\"{1}\"\r\nContent-Type: {2}\r\n\r\n",
                        boundary,
                        param.Key,
                        "application/octet-stream");
                    formDataStream.Write(Encoding.UTF8.GetBytes(postData), 0, Encoding.UTF8.GetByteCount(postData));

                    // Write the file data directly to the Stream, rather than serializing it to a string.
                    formDataStream.Write((param.Value as byte[]), 0, (param.Value as byte[]).Length);
                }
                else
                {
                    string postData = string.Format("--{0}\r\nContent-Disposition: form-data; name=\"{1}\"\r\n\r\n{2}",
                        boundary,
                        param.Key,
                        param.Value);
                    formDataStream.Write(Encoding.UTF8.GetBytes(postData), 0, Encoding.UTF8.GetByteCount(postData));
                }
            }

            // Add the end of the request.  Start with a newline
            string footer = "\r\n--" + boundary + "--\r\n";
            formDataStream.Write(Encoding.UTF8.GetBytes(footer), 0, Encoding.UTF8.GetByteCount(footer));

            // Dump the Stream into a byte[]
            formDataStream.Position = 0;
            byte[] formData = new byte[formDataStream.Length];
            formDataStream.Read(formData, 0, formData.Length);
            formDataStream.Close();

            return formData;
        }
    }
}

