/*
 * © 2017 LogMeIn, Inc. All Rights Reserved.
 * All rights reserved.
 * 
 * This software is distributed under the terms and conditions of the
 * LogMeIn SDK License Agreement. Please see file LICENSE for details.
 * 
 * Auto-generated file.
 */


using System;
using System.Collections.Generic;
using LogMeIn.GoToTraining.Api.Common;
using LogMeIn.GoToTraining.Api.Model;


namespace LogMeIn.GoToTraining.Api
{
    /// <summary>
    ///
    /// </summary>
    public class RecordingsApi
    {
        private string basePath = "https://api.getgo.com/G2T/rest";
        private readonly ApiInvoker apiInvoker = ApiInvoker.GetInstance();

        /// <summary>
        /// Initializes a new instance of the RecordingsApi class using the
        /// default endpoint base url for the services being accessed.
        /// </summary>
        public RecordingsApi()
        {
        }

        /// <summary>
        /// Initializes a new instance of the RecordingsApi class using an endpoint base
        /// url other than the default. Use this constructor only if you should need
        /// to override the default endpoint base url.
        /// </summary>
        /// <param name="basePath">The endpoint base url for the services being accessed.</param>
        public RecordingsApi(String basePath)
        {
            this.basePath = basePath;
        }

        /// <summary>
        /// Gets the API invoker
        /// </summary>
        public ApiInvoker Invoker
        {
            get { return apiInvoker; }
        }

        /// <summary>
        /// Gets the endpoint base url for the services being accessed
        /// </summary>
        public String BasePath
        {
            get { return basePath; }
        }

        /// <summary>
        /// Get Download for Online Recordings
        /// This call provides the download for the given recording by returning a 302 redirect to the original file.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <param name="recordingId">the unique id of the recording</param>
        /// <returns>void</returns>
        /// <exception cref="ApiException"></exception>
        public void getRecordingDownloadById(string authorization, long? trainingKey, long? recordingId)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            if (recordingId == null)
                throw new ApiException("Required parameter recordingId is null.");

            // create path and map variables
            var path = "/trainings/{trainingKey}/recordings/{recordingId}"
                .Replace("{format}", "json")
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()))
                .Replace("{" + "recordingId" + "}", apiInvoker.EscapeString(recordingId.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            apiInvoker.InvokeAPI(basePath, path, "GET", queryParams, null, headerParams, formParams, typeof(void), contentType);
        }

        /// <summary>
        /// Get Online Recordings for Training
        /// This call retrieves information on all online recordings for a given training. If there are none, it returns an empty list.
        /// </summary>
        /// <param name="authorization">Access token</param>
        /// <param name="trainingKey">The key of the training</param>
        /// <returns>RecordingsListForTraining</returns>
        /// <exception cref="ApiException"></exception>
        public RecordingsListForTraining getRecordingsForTraining(string authorization, long? trainingKey)
        {
            if (authorization == null)
                throw new ApiException("Required parameter authorization is null.");

            if (trainingKey == null)
                throw new ApiException("Required parameter trainingKey is null.");

            // create path and map variables
            var path = "/trainings/{trainingKey}/recordings"
                .Replace("{format}", "json")
                .Replace("{" + "trainingKey" + "}", apiInvoker.EscapeString(trainingKey.ToString()));

            // query params
            var queryParams = new Dictionary<String, String>();
            var headerParams = new Dictionary<String, String>();
            var formParams = new Dictionary<String, object>();

            headerParams.Add("Authorization", authorization);

            var contentTypes = new List<string> { "application/json" };
            string contentType = contentTypes.Count > 0 ? contentTypes[0] : "application/json";

            var response = apiInvoker.InvokeAPI(basePath, path, "GET", queryParams, null, headerParams, formParams, typeof(RecordingsListForTraining), contentType);
            if (response != null)
                return (RecordingsListForTraining)response;

            return null;
        }


    }
}
